﻿using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE;
using EnvDTE80;
using VsWebSite;

namespace PHVGenerator
{
    class DatabaseUtilities : IPHVGeneration
    {
        DTE2 applicationObject;
        AddIn addInInstance;
        ProjectItem codeFolder;
        Solution2 sln;
        bool IsStatic;
        bool IsCustomized;
        GenerationUtilities util;
        ProjectItem pjic;
        struct CodeClasses
        {
            public CodeClass2 cc;
            public CodeClass2 cg;
        }

        public DatabaseUtilities(DTE2 ApplicationObject, AddIn AddInInstance)
        {
            applicationObject = ApplicationObject;

            sln = (Solution2) ApplicationObject.Solution;
            addInInstance = AddInInstance;
            util = new GenerationUtilities(AddInInstance, applicationObject,"ConnectionStringGenerator");
            IsStatic = true;
            IsCustomized = true;
        }

        public void GenerateConnectionManager(ProjectInfo prjInfo)
        {
            ProjectItem prji = null;
            try
            {
                prji = prjInfo.Project.ProjectItems.Item("app.config");
            }
            catch
            {
                try
                {
                    prji = prjInfo.Project.ProjectItems.Item("web.config");
                }
                catch { }
            }
            if (prji != null)
            {
                GenerateConnectionManager(prji.Document, prjInfo);
            }
        }

        public void GenerateConnectionManager(Document Doc, ProjectInfo prjInfo)
        {
            if (prjInfo.Project == null)
            {
             return;
            }                        
        
            if (!CheckForConnectionStrings(prjInfo))
            {
                return;
            }
            
            AddGenerationFolder(prjInfo);
            CodeClasses ccs = AddConnectionManagerClasses(prjInfo);
            if (ccs.cc == null)
            {
                util.WriteOutput("Unable to create Connection Manager.", GenerationLevel.severe);
                return;
            }
            
            AddGeneratedCode(ccs.cg, prjInfo);
            if (IsCustomized)
            {
                AddCustomizedCode(ccs.cc, prjInfo);
            }
        }

        private bool CheckForConnectionStrings(ProjectInfo pi)
        {
            ProjectItem cfg = null;
            try
            {
                cfg = pi.Project.ProjectItems.Item("web.config");
            }
            catch
            {
                try
                {
                    cfg = pi.Project.ProjectItems.Item("app.config"); 
                }
                catch {}
            }
            if (cfg == null)
            {
                return false;
            }

            System.Xml.XmlDocument dom = GetConfigXML(cfg);
            System.Xml.XmlNode ndCons = dom.SelectSingleNode("//connectionStrings");
            if (ndCons == null || ndCons.ChildNodes.Count == 0)
            {
                return false;
            }
            return true;
        }

        void AddGeneratedCode(CodeClass2 cc, ProjectInfo prjInfo)
        {
            util.WriteOutput("Adding code", GenerationLevel.information);
            if (IsStatic)
            {
                try
                {
                    cc.IsShared = true;
                }
                catch { }
            }
            
            ProjectItem cfg;
            try
            {
                cfg = prjInfo.Project.ProjectItems.Item("web.config");
            }
            catch
            {
                cfg = prjInfo.Project.ProjectItems.Item("app.config");
            }

            System.Xml.XmlDocument dom;
            dom = GetConfigXML(cfg);
            System.Xml.XmlNode ndCons = dom.SelectSingleNode("//connectionStrings");
            foreach (System.Xml.XmlNode ndCon in ndCons)
            {
                CodeProperty cp;
                string PropertyName = ndCon.Attributes["name"].Value;
                util.WriteOutput("Adding property " + PropertyName, GenerationLevel.information);
                cp =   cc.AddProperty(PropertyName, null, vsCMTypeRef.vsCMTypeRefString, -1, vsCMAccess.vsCMAccessPublic, null);
                if (IsStatic)
                {
                    try
                    {
                        ((CodeProperty2) cp).IsShared = true;
                    }
                    catch
                    {
                        cp.Getter.IsShared = true;
                    }
                }
                EditPoint epGetter = cp.Getter.GetStartPoint(vsCMPart.vsCMPartBody).CreateEditPoint();
                epGetter.Delete(cp.Getter.GetEndPoint(vsCMPart.vsCMPartBody));
                String CodeLine = "";
                
                if (prjInfo.ProjectLanguage == "CSharp")
                {
                    epGetter.Indent(epGetter, 1);
                    if (!IsCustomized)
                    {
                        CodeLine = "return System.Configuration.ConfigurationManager.ConnectionStrings[\""
                                        + PropertyName + "\"].ConnectionString;\r\n\t\t\t";
                    }
                    else
                    {
                        CodeLine = "return " + PropertyName + "Customization(System.Configuration.ConfigurationManager.ConnectionStrings[\""
                                         + PropertyName + "\"].ConnectionString);\r\n\t\t\t";
                    }
                }
                if (prjInfo.ProjectLanguage == "VisualBasic")
                {
                    epGetter.Indent(epGetter, 3);
                    if (!IsCustomized)
                    {
                        CodeLine = "return System.Configuration.ConfigurationManager.ConnectionStrings(\""
                                         + PropertyName + "\").ConnectionString \r\n\t\t";
                    }
                    else
                    {
                        CodeLine = "return " + PropertyName + "Customization(System.Configuration.ConfigurationManager.ConnectionStrings(\""
                                      + PropertyName + "\").ConnectionString) \r\n\t\t";
                    }
                }
                epGetter.Insert(CodeLine);                
            }
            util.WriteOutput("Code added", GenerationLevel.information);
        }

        void AddCustomizedCode(CodeClass2 cc, ProjectInfo prjInfo)
        {
            util.WriteOutput("Adding custom code", GenerationLevel.information);
            if (IsStatic)
            {
                try
                {
                    cc.IsShared = true;
                }
                catch { }
            }

            ProjectItem cfg;
            try
            {
                cfg = prjInfo.Project.ProjectItems.Item("web.config");
            }
            catch
            {
                cfg = prjInfo.Project.ProjectItems.Item("app.config");
            }

            System.Xml.XmlDocument dom = GetConfigXML(cfg);
            System.Xml.XmlNode ndCons = dom.SelectSingleNode("//connectionStrings");
            foreach (System.Xml.XmlNode ndCon in ndCons)
            {
                EnvDTE.CodeFunction cf;
                string FunctionName = ndCon.Attributes["name"].Value + "Customization";
                util.WriteOutput("Adding method " + FunctionName, GenerationLevel.information);
                try
                {
                    CodeElement ce = cc.Members.Item(FunctionName);
                }
                catch
                {
                    cf = cc.AddFunction(FunctionName, vsCMFunction.vsCMFunctionFunction, "System.String", -1, vsCMAccess.vsCMAccessPrivate, null);
                    if (IsStatic)
                    {
                        ((CodeFunction)cf).IsShared = true;
                    }                    
                    cf.AddParameter("ConnectionString", "System.String", -1);
                    
                    EditPoint epFunction = cf.GetStartPoint(vsCMPart.vsCMPartBody).CreateEditPoint();
                    epFunction.Delete(cf.GetEndPoint(vsCMPart.vsCMPartBody));
                    String CodeLine = "";
                    switch (prjInfo.ProjectLanguage)
                    {
                        case "CSharp":
                            CodeLine = "\t\t\t return ConnectionString; \r\n";
                            break;
                        case "VisualBasic":
                            CodeLine = "return ConnectionString \r\n";
                            break;
                    }
                    EditPoint ep = cf.GetStartPoint(vsCMPart.vsCMPartBody).CreateEditPoint();
                    ep.Insert("\r\n"); 
                    ep.Insert(CodeLine);                    
                }
            }
            util.WriteOutput("Code added", GenerationLevel.information);
        }

        CodeClass2 CustomizeTemplate(FileCodeModel fcm, ProjectInfo prjInfo)
        {
            CodeElement2 codeClass = null;
            util.WriteOutput("Customizing template", GenerationLevel.information);
            string namespaceName;
            try
            {
                namespaceName = prjInfo.Project.Properties.Item("DefaultNamespace").Value.ToString();
            }
            catch
            {
                namespaceName = "";
            }

            foreach (CodeElement2 ce in fcm.CodeElements)
            {    
             if (ce.Kind == vsCMElement.vsCMElementNamespace)
             {
                    util.WriteOutput("Modifying namespace", GenerationLevel.information);
                    ce.Name = namespaceName;
                    CodeElement2 ceNamespace = (CodeElement2)fcm.CodeElements.Item(namespaceName);
                    foreach (CodeElement2 ceClass in ceNamespace.Children)
                    {
                        if (ceClass.Kind == vsCMElement.vsCMElementClass)
                        {
                            codeClass = ceClass;
                        }
                    }
                }
                if (ce.Kind == vsCMElement.vsCMElementClass)
                {
                    codeClass = ce;
                }
            }
            
            ((CodeClass2) codeClass).ClassKind = EnvDTE80.vsCMClassKind.vsCMClassKindPartialClass;

            foreach (CodeElement2 ce in codeClass.Children)
            {
             if (ce.Kind == vsCMElement.vsCMElementFunction &&
                 ce.Name == "ConnectionManager")
             {
              util.WriteOutput("Removing constructor", GenerationLevel.information);
              fcm.Remove(ce);
             }
            }

            if (prjInfo.Project.Kind != "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
            {
             util.WriteOutput("Adding reference to Configuration", GenerationLevel.information);
             VSLangProj.VSProject vsPrj;
                
             vsPrj = (VSLangProj.VSProject)prjInfo.Project.Object;
             vsPrj.References.Add("System.Configuration");             
            }
            return (CodeClass2)codeClass;
        }

        FileCodeModel GetFileCodeModel(ProjectItem ConnMgr, ProjectInfo prjInfo)
        {
            FileCodeModel fcm;
            
            util.WriteOutput("Retrieving FileCodeModel", GenerationLevel.information);
            if (prjInfo.Project.Kind == "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
            {
                VsWebSite.VSWebProjectItem tmpWPI;
                tmpWPI = (VsWebSite.VSWebProjectItem)ConnMgr.Object;
                tmpWPI.Load();
                fcm = tmpWPI.ProjectItem.FileCodeModel;
                VSWebSite vsw = (VSWebSite)prjInfo.Project.Object;
                vsw.CodeFolders.Add("App_Code");
            }
            else
            {
                fcm = ConnMgr.FileCodeModel;
            }
            return fcm;
        }

        void AddGenerationFolder(ProjectInfo prjInfo)
        {
            util.WriteOutput("Adding code folder", GenerationLevel.information);
            if (prjInfo.Project.Kind == "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
         {
          try
          {
           prjInfo.Project.ProjectItems.AddFolder("App_Code", "{6BB5F8EF-4483-11D3-8BCF-00C04F8EC28C}");
          }
          catch { };
          codeFolder = prjInfo.Project.ProjectItems.Item("App_Code");
         }
         else
         {
          try
          {
           prjInfo.Project.ProjectItems.AddFolder("Generated Code", Constants.vsProjectItemKindPhysicalFolder);
          }
          catch {};
          codeFolder = prjInfo.Project.ProjectItems.Item("Generated Code");
         }            
        }

        CodeClasses AddConnectionManagerClasses(ProjectInfo prjInfo)
        {          
         CodeClasses ccs = new CodeClasses();
         string ProjectPath;
         util.WriteOutput("Adding code file", GenerationLevel.information);
         ProjectPath = System.IO.Path.GetDirectoryName(prjInfo.Project.FullName);
        
         string ItemTemplatePath;
         FileCodeModel fcm;

         if (IsCustomized)
         {
             pjic = sln.FindProjectItem(@ProjectPath + @"\" + codeFolder.Name + @"\ConnectionManager.Customization" + prjInfo.FileType);
             if (pjic == null)
             {
                 if (prjInfo.Project.Kind == "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
                 {
                     ItemTemplatePath = sln.GetProjectItemTemplate("Class.zip", @"Web\" + prjInfo.ProjectLanguage);
                 }
                 else
                 {
                     ItemTemplatePath = sln.GetProjectItemTemplate("Class.zip", prjInfo.ProjectLanguage);
                 }
                 pjic = codeFolder.ProjectItems.AddFromTemplate(ItemTemplatePath, "ConnectionManager.Customization" + prjInfo.FileType);
             }
             pjic = sln.FindProjectItem(@ProjectPath + @"\" + codeFolder.Name + @"\ConnectionManager.Customization" + prjInfo.FileType);
             fcm = GetFileCodeModel(pjic, prjInfo);
             ccs.cc  = CustomizeTemplate(fcm, prjInfo);             
         }

         ProjectItem pji = sln.FindProjectItem(@ProjectPath + @"\" + codeFolder.Name + @"\ConnectionManager.Generation" + prjInfo.FileType);
         if (pji != null)
         {
          util.WriteOutput("Deleting existing file", GenerationLevel.information);
          try
          {
              pji.Delete();
          }
          catch (Exception ex) 
          { 
              string msg = ex.Message;
              util.WriteOutput("Error Deleting existing file: " + ex.Message, GenerationLevel.information);
          }
         }
         if (prjInfo.Project.Kind == "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
         {
             ItemTemplatePath = sln.GetProjectItemTemplate("Class.zip", @"Web\" + prjInfo.ProjectLanguage);
         }
         else
         {
             ItemTemplatePath = sln.GetProjectItemTemplate("Class.zip", prjInfo.ProjectLanguage);
         }
         pji = codeFolder.ProjectItems.AddFromTemplate(ItemTemplatePath, "ConnectionManager.Generation" + prjInfo.FileType);

         if (pji == null)
         {
             pji = sln.FindProjectItem(@ProjectPath + @"\" + codeFolder.Name + @"\ConnectionManager.Generation" + prjInfo.FileType);
         }
         util.WriteOutput("File added", GenerationLevel.information);
         fcm = GetFileCodeModel(pji, prjInfo);
            
         ccs.cg = CustomizeTemplate(fcm, prjInfo);

         return ccs;
        }   
      
    
        public static string MenuItemName
        {
            get
            {
             return "CnstrGentr";
            }
        }

        public static string MenuItemCaption
        {
            get
            {
             return "Connection Manager";
            }
        }
        public static string MenuToolTip
        {
            get
            {
             return "Create a class for managing connection strings";
            }
        }

        System.Xml.XmlDocument GetConfigXML(ProjectItem cfg)
        {
            System.Xml.XmlDocument dom;
            if (cfg.Document != null)
            {
                TextDocument td = ((TextDocument)cfg.Document.Object("TextDocument"));
                EditPoint ep = td.StartPoint.CreateEditPoint();
                dom = new System.Xml.XmlDocument();
                dom.LoadXml(ep.GetText(td.EndPoint));
            }
            else
            {
                dom = new System.Xml.XmlDocument();
                dom.Load(@cfg.Properties.Item("FullPath").Value.ToString());
            }
            return dom;
        }
      }    
   }

