﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EnvDTE;
using EnvDTE80;
using EnvDTE90;
using VsWebSite;
using Microsoft.VisualStudio.CommandBars;

namespace PHVGenerator
{
    public enum GenerationLevel
    {
        information,
        severe
    }

    public struct ProjectInfo
    {
        public string ProjectLanguage;
        public string FileType;
        public Project Project;
    }        
    public class GenerationUtilities
    {
        DTE2 applicationObject;
        string subCategory;
        private AddIn addInInstance;

        public GenerationUtilities(AddIn AddInInstance, DTE2 ApplicationObject, string SubCategory)
        {
            
            applicationObject = ApplicationObject;
            subCategory = SubCategory;
            addInInstance = AddInInstance;
            TaskList tl = applicationObject.ToolWindows.TaskList;
            foreach (TaskItem ti in tl.TaskItems)
            {
                if (ti.SubCategory == SubCategory)
                {
                    ti.Delete();
                }
            }

        }    

        public void WriteOutput(string Message, GenerationLevel Level)
        {
            if (Level == GenerationLevel.severe)
            {
                TaskList tl = applicationObject.ToolWindows.TaskList;
                TaskItems2 tis = (TaskItems2)tl.TaskItems;
                TaskItem ti = tis.Add2("CodeGeneration",
                              subCategory,
                              Message,
                              (int)vsTaskPriority.vsTaskPriorityHigh,
                              (int)vsTaskIcon.vsTaskIconCompile,
                               true,
                              "",
                              0,
                              true,
                              true,
                              false);
            }
            else if (Level == GenerationLevel.information)
            {
                applicationObject.DTE.StatusBar.Text = Message;
            }
          }
        public static string GetValue(string Name)
        {
            Microsoft.Win32.RegistryKey key;

            key = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(
                             @"SOFTWARE\Microsoft\VisualStudio\9.0", false);
            return key.GetValue(Name, "").ToString();
        }

        public static void SaveValue(string Name, string value)
        {
            Microsoft.Win32.RegistryKey key;

            key = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(
                                @"SOFTWARE\Microsoft\VisualStudio\9.0", true);
            key.SetValue(Name, value,
                     Microsoft.Win32.RegistryValueKind.String);
        }

        public ProjectItem AddFolderAndFile(ProjectInfo pi, string TemplateName, string FileName)
        {
           ProjectItems pjis = pi.Project.ProjectItems;
           WriteOutput("Adding code folder", GenerationLevel.information);

           if (pi.Project.Kind == "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
           {
               VSWebSite vsw;
               vsw = (VSWebSite)pi.Project.Object;
               if (vsw.CodeFolders.Item("App_Code") == null)
               {
                   vsw.CodeFolders.Add("App_Code/GeneratedCode");
               }
               return (ProjectItem) vsw.CodeFolders.Item(@"App_Code\GeneratedCode");
           }
           else
           {
               try
               {
                   pjis.AddFolder("Generated Code", Constants.vsProjectItemKindPhysicalFolder);
               }
               catch { };
               ProjectItem CodeFolder = pjis.Item("Generated Code");

               if (FileName == "Default")
               {
                   FileName = "_Default";
               }
               try
               {
                   return CodeFolder.ProjectItems.Item(FileName + ".generation" + pi.FileType);
               }
               catch
               {
                   string ItemTemplatePath = ((Solution2)applicationObject.Solution).GetProjectItemTemplate(TemplateName + pi.ProjectLanguage + ".zip", pi.ProjectLanguage);
                   CodeFolder.ProjectItems.AddFromTemplate(ItemTemplatePath, FileName);
                   return CodeFolder.ProjectItems.Item(FileName + ".generation" + pi.FileType);
               }
           }
        }
        
        public static ProjectInfo SetProjectInformation(Document Doc, DTE2 applicationObject)
        {
            ProjectInfo pi = new ProjectInfo();
            ProjectItem pji;
            //if no document is passed, try to find the active document or the project
            //for the currently selected item
            if (Doc == null)
            {
               if (applicationObject.ActiveDocument != null)
               {
                   Doc = applicationObject.ActiveDocument;
               }
               else
               {
                   try
                   {
                    pi.Project = applicationObject.SelectedItems.Item(1).Project;                   
                   }
                   catch{}
               }
            }

            //if we don't have a document or a project, give up
            if (Doc == null && pi.Project == null)
            {
                return pi;
            }
            
            //if we have a document but don't have a project, try to derive
            // the project from the document
            if (Doc != null && pi.Project == null)
                try
                {
                    pji = Doc.ProjectItem;
                    pi.Project = Doc.ProjectItem.ContainingProject;
                }
                catch
                {
                    foreach (Project prj in applicationObject.Solution.Projects)
                    {
                        if (System.IO.Path.GetDirectoryName(Doc.FullName) == System.IO.Path.GetDirectoryName(prj.FullName))
                        {
                            pi.Project = prj;
                            break;
                        }
                    }
                }
            //if we still don't have a project, give up
            if (pi.Project == null)
            {
                return pi;
            }
            
            //fill out the rest of the project information
            switch (pi.Project.Kind)
            {
                //c# projects
                case "{F184B08F-C81C-45F6-A57F-5ABD9991F28F}":
                    pi.ProjectLanguage = "VisualBasic";
                    pi.FileType = ".vb";
                    break;
                //VB projects
                case "{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}":
                    pi.ProjectLanguage = "CSharp";
                    pi.FileType = ".cs";
                    break;
                //Web Site projects
                case "{E24C65DC-7377-472b-9ABA-BC803B73C61A}":
                    switch (pi.Project.Properties.Item("CurrentWebsiteLanguage").Value.ToString())
                    {
                        case "Visual C#":
                            pi.ProjectLanguage = "CSharp";
                            pi.FileType = ".cs";
                            break;
                        case "Visual Basic":
                            pi.ProjectLanguage = "VisualBasic";
                            pi.FileType = ".vb";
                            break;
                    }
                    break;
            }
            return pi;
        }

        internal CommandBar GetMenu(string proposedMenuName)
        {
            string FoundMenuName;
            try
            {
                System.Resources.ResourceManager resourceManager =
                  new System.Resources.ResourceManager(
                     addInInstance.ProgID + ".CommandBar",
                     System.Reflection.Assembly.GetExecutingAssembly());
                System.Globalization.CultureInfo cultureInfo = new
                     System.Globalization.CultureInfo(applicationObject.LocaleID);
                if (cultureInfo.TwoLetterISOLanguageName == "zh")
                {
                    System.Globalization.CultureInfo parentCultureInfo =
                                                              cultureInfo.Parent;
                    FoundMenuName = resourceManager.GetString(
                             String.Concat(parentCultureInfo.Name, proposedMenuName));
                }
                else
                {
                    FoundMenuName = resourceManager.GetString(String.Concat(
                                     cultureInfo.TwoLetterISOLanguageName, proposedMenuName));
                }
            }
            catch
            {
                FoundMenuName = proposedMenuName;
            }

            if (FoundMenuName == "")
            {
                FoundMenuName = proposedMenuName;
            }
            CommandBars cbs = (CommandBars) applicationObject.CommandBars;
            return cbs[FoundMenuName];
        }

        internal void AddToSubMenu(CommandBar cb, Commands2 cmds, ref Object[] contextGUIDS,
                          string menuItemName, string menuItemCaption, string menuToolTip)
        {

            Command NamedCommand = null;
            try
            {
                //System.Windows.Forms.MessageBox.Show("Finding NamedCommand");

                NamedCommand = applicationObject.Commands.Item(
                             addInInstance.ProgID + "." + menuItemName, 1);
                //System.Windows.Forms.MessageBox.Show("Finding NamedCommand done: " + NamedCommand.Name);
                //System.Windows.Forms.MessageBox.Show("Finding NamedCommand done: " + NamedCommand.ID);
                //System.Windows.Forms.MessageBox.Show("Finding NamedCommand done: " + _addInInstance.ProgID + "." + menuItemName);                                       
            }
            catch
            {
                //System.Windows.Forms.MessageBox.Show("Creating NamedCommand");
                NamedCommand = cmds.AddNamedCommand2(addInInstance,
                               menuItemName, menuItemCaption, menuToolTip,
                               true, 50, ref contextGUIDS,
                              (int)vsCommandStatus.vsCommandStatusSupported +
                              (int)vsCommandStatus.vsCommandStatusEnabled,
                              (int)vsCommandStyle.vsCommandStylePictAndText,
                              vsCommandControlType.vsCommandControlTypeButton);
                //System.Windows.Forms.MessageBox.Show("Creating NamedCommand done");
            }

            try
            {
                //System.Windows.Forms.MessageBox.Show("Finding menu");                   
                CommandBarControl cbc =
                              cb.Controls[menuItemCaption];
                //System.Windows.Forms.MessageBox.Show("Finding menu done");
            }
            catch
            {
                //System.Windows.Forms.MessageBox.Show("Adding command to menu");
                NamedCommand.AddControl(cb, 1);
                //System.Windows.Forms.MessageBox.Show("Adding command to menu done");
            }
        }

    }
  }
