﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using EnvDTE;
using EnvDTE80;
using System.Xml;
using System.CodeDom;
using System.CodeDom.Compiler;
using VsWebSite;
using VsWebSite90;

namespace PHVGenerator 
{
    
    class WebFormUtilities : IPHVGeneration
    {
        readonly DTE2 applicationObject;
        readonly AddIn addInInstance;
        readonly GenerationUtilities cgu;
        
        struct connParms
        {
            public CodeTypeReference connType;
            public CodeTypeReference cmdType;
            public string parmPrefix;
            public CodeFieldReferenceExpression parmType;
        }
        struct Classes
        {
            public CodeClass lookupClass;
            public CodeClass masterClass;
        }
           
        public static string MenuItemName
        {
            get
            {
             return "lkupvdGentr";
            }
        }

        public static string MenuItemCaption
        {
            get
            {
             return "Lookup Validator";
            }
        }
        public static string MenuToolTip
        {
            get
            {
             return "Create a class for integrating with an ASP.NET validator";
            }
        }          
        
        public  WebFormUtilities(DTE2 ApplicationObject, AddIn AddInInstance)
        {
            applicationObject = ApplicationObject;
            addInInstance = AddInInstance;
            //System.Windows.Forms.MessageBox.Show("starting WebFormUtilities");
            cgu = new GenerationUtilities(AddInInstance, ApplicationObject, "Lookup Validator");
        }

        public void GenerateValdiator(ProjectInfo pi)
        {
            //System.Windows.Forms.MessageBox.Show("Calling generate validator without document");
            if (applicationObject.ActiveWindow.Document.Name.EndsWith(".aspx"))
            {
                GenerateValidator(null, pi);
            }
        }

        public void GenerateValidator(Document Doc, ProjectInfo pi)
        {
            //System.Windows.Forms.MessageBox.Show("In generate validator");
            bool isNewWindow = false;
            if (Doc == null)
            {
                //System.Windows.Forms.MessageBox.Show("Getting projectitem");
                ProjectItem pji = null;
                //System.Windows.Forms.MessageBox.Show("Using Activewindow");
                pji = applicationObject.ActiveDocument.ProjectItem;
                //System.Windows.Forms.MessageBox.Show("Got projectitem");
                if (pji.Document != null && pji.Document.Windows.Count > 0)
                {
                    //System.Windows.Forms.MessageBox.Show("Getting window");
                    Doc = pji.Document.Windows.Item(1).Document;
                }
                else
                {
                    //System.Windows.Forms.MessageBox.Show("Opening new window");
                    isNewWindow = true;
                    Window win = pji.Open(Constants.vsViewKindTextView);
                    Doc = win.Document;
                }
            }
            //System.Windows.Forms.MessageBox.Show("Finished finding window"); 
                       
            TextDocument td = (TextDocument)Doc.Object("TextDocument");

            EditPoint edStart = td.StartPoint.CreateEditPoint();
            EditPoint edFound = null;
            TextRanges txts = null;
            //System.Windows.Forms.MessageBox.Show("Looking for validators"); 
            if (edStart.FindPattern(":LookupValidator", (int)
                 vsFindOptions.vsFindOptionsFromStart, ref edFound, ref txts))
            {
                //System.Windows.Forms.MessageBox.Show("Found at least one");
                string code = ProcessValidators(td, pi);
                //System.Windows.Forms.MessageBox.Show("ready to insert");
                if (code != "")
                {
                  bool CreatedUndo = false;
                  UndoContext undo = applicationObject.UndoContext;

                  try
                  {
                      if (!undo.IsOpen)
                      {
                          undo.Open("LookupValidator Generate", true);
                          CreatedUndo = true;
                      }
                      string FileName = Doc.Name.Substring(0, Doc.Name.IndexOf("."));
                      if (pi.Project.Kind == "{E24C65DC-7377-472b-9ABA-BC803B73C61A}")
                      {
                          //System.Windows.Forms.MessageBox.Show("Adding website");             
                          AddCodeWebSite(code, Doc);
                      }
                      else
                      {
                          //System.Windows.Forms.MessageBox.Show("Adding project");
                          ProjectItem CodeFile = cgu.AddFolderAndFile(pi, "LookupValidatorBase", FileName);
                          AddCode(code, CodeFile);
                      }
                  }
                  finally
                  {
                      if (undo.IsOpen && CreatedUndo)
                      {
                          undo.Close();
                      }
                  }
                }
            }
            //System.Windows.Forms.MessageBox.Show("finished generation");
            if (isNewWindow)
            {
                Doc.Windows.Item(1).Close(vsSaveChanges.vsSaveChangesYes);
                if (!Doc.Saved)
                {
                    GenerationUtilities cgu = new GenerationUtilities(addInInstance, applicationObject, "Lookup Validator");
                    cgu.WriteOutput("Unable to save changes", GenerationLevel.severe);
                }
            }
        }

        void AddCodeWebSite(string code, Document Doc)
        {
         //System.Windows.Forms.MessageBox.Show("In add website");
         VSWebProjectItem2 vspji =  (VSWebProjectItem2) Doc.ProjectItem.Object;
         ProjectItem codeFile = vspji.RelatedFiles.Item(1);
         VSWebProjectItem2 vsCodeFile = (VSWebProjectItem2) codeFile.Object;
         vsCodeFile.Load();
         FileCodeModel fcm = vsCodeFile.ProjectItem.FileCodeModel;

         Classes cls = FindClass(fcm);

         //System.Windows.Forms.MessageBox.Show("Fix class name");
         if (cls.lookupClass == null)
         {
             string className;
             className = Doc.Name.Substring(0,Doc.Name.IndexOf("."));
             if (className == "Default")
             {
                 className = "_Default";
             }
             else
             {
                 className = Doc.Name;
             }
             //System.Windows.Forms.MessageBox.Show("Adjusted name");
             string tempClassName = className + "_New";
             object[] bases = { "System.Web.UI.Page" };
             //System.Windows.Forms.MessageBox.Show("about to add class: " + tempClassName);
             try
             {
                 cls.lookupClass = fcm.AddClass(tempClassName, -1, bases, null, vsCMAccess.vsCMAccessPublic);
             }
             catch 
             {
                 EditPoint codePoint = cls.masterClass.GetEndPoint(vsCMPart.vsCMPartWhole).CreateEditPoint();
                 codePoint.Insert("\r\n \r\n Public Class " + tempClassName + " \r\n End Class");
                 cls = FindClass(fcm);
             }
             //System.Windows.Forms.MessageBox.Show("Adjust properties");
             CodeClass2 lookupClass2 = (CodeClass2)cls.lookupClass;
             lookupClass2.ClassKind = EnvDTE80.vsCMClassKind.vsCMClassKindPartialClass;
             lookupClass2.Name = className;
         }

         //System.Windows.Forms.MessageBox.Show("find methods");
         foreach (CodeElement md in cls.lookupClass.Children)
         {
          if (md.Kind == vsCMElement.vsCMElementFunction)
          {
           if (code.Contains(md.Name))
           {
            cls.lookupClass.RemoveMember(md);
           }
          }
         }
         
         //System.Windows.Forms.MessageBox.Show("insert code");         
         EditPoint codePoint2 = cls.lookupClass.GetEndPoint(vsCMPart.vsCMPartBody).CreateEditPoint();
         codePoint2.Insert(code.Trim() + " \r\n ");
         //vsCodeFile.Unload();
         //System.Windows.Forms.MessageBox.Show("code added");         
        }
        
        string ProcessValidators( TextDocument td, ProjectInfo pi)
        {   
            EditPoint edStart = td.StartPoint.CreateEditPoint();
            EditPoint edEndFound = null;
            TextRanges txts = null;
            StringBuilder code = new StringBuilder();
            //System.Windows.Forms.MessageBox.Show("Look for validator");    
            while (edStart.FindPattern("LookupValidator", (int)
                 vsFindOptions.vsFindOptionsNone, ref edEndFound, ref txts))
            {
                //System.Windows.Forms.MessageBox.Show("In loop");
                EditPoint edElementEnd = null;
                edEndFound.FindPattern(">", (int)
                     vsFindOptions.vsFindOptionsNone, ref edElementEnd, ref txts);
                string element = "<" + edStart.GetText(edElementEnd) + "</LookupValidator>";
                
                XmlDocument xdoc = new XmlDocument();
                xdoc.LoadXml(element);
                XmlNode nd = xdoc.ChildNodes[0];
                //System.Windows.Forms.MessageBox.Show("Loading tag");
                if (nd.Attributes["SuppressGeneration"] == null || nd.Attributes["SuppressGeneration"].Value == "false")
                {
                    string Id = nd.Attributes["ID"].Value;
                    string ConnectionName = "";
                    string TableName = "";
                    string FieldName = "";
                    string DType = "";
                    if (nd.Attributes["ConnectionStringName"] != null)
                    {
                        ConnectionName = nd.Attributes["ConnectionStringName"].Value;
                    }
                    if (nd.Attributes["TableName"] != null)
                    {
                        TableName = nd.Attributes["TableName"].Value;
                    }
                    if (nd.Attributes["FieldName"] != null)
                    {
                        FieldName = nd.Attributes["FieldName"].Value;
                    }
                    if (nd.Attributes["Type"] != null)
                    {
                        DType = nd.Attributes["Type"].Value;
                    }
                    else
                    {
                        DType = "Integer";
                    }
                    if (ConnectionName != "" && TableName != "" && FieldName != "" && DType != "")
                    {
                        string EventName;
                        if (nd.Attributes["onServerValidate"] == null)
                        {
                            edElementEnd.CharLeft(1);
                            string eventAttribute;
                            EventName = Id + "_ServerValidate";
                            eventAttribute = "onServerValidate=\"" + EventName + " \"";
                            edElementEnd.Insert(" " + eventAttribute + " ");
                        }
                        else
                        {
                            EventName = nd.Attributes["onServerValidate"].Value;
                        }
                        //System.Windows.Forms.MessageBox.Show("Ready to generate");
                        //System.Windows.Forms.MessageBox.Show("EN:" + EventName);
                        //System.Windows.Forms.MessageBox.Show("Id:" + Id);
                        //System.Windows.Forms.MessageBox.Show("CN:" + ConnectionName);
                        //System.Windows.Forms.MessageBox.Show("TN:" + TableName);
                        //System.Windows.Forms.MessageBox.Show("TN:" + FieldName);
                        //System.Windows.Forms.MessageBox.Show("DT:" + DType);
                        //System.Windows.Forms.MessageBox.Show("FT:" + pi.FileType);
                        //System.Windows.Forms.MessageBox.Show("About to call generator");
                        code.Append(GenerateCode(EventName, Id, ConnectionName, TableName, FieldName, DType, pi));
                        //System.Windows.Forms.MessageBox.Show("Got code: " + code.ToString());
                    }
                }
                edElementEnd.FindPattern("LookupValidator>", (int)
                     vsFindOptions.vsFindOptionsNone, ref edStart, ref txts);                
            }
            return code.ToString();
        }

        string GenerateCode(string EventName, string Id, string ConnectionName, string TableName, string FieldName, string DType, ProjectInfo pi)
        {
            //System.Windows.Forms.MessageBox.Show("1");
            //create an array of CodeExpression parameters to use with methods
            CodeExpression[] parms1 = new CodeExpression[1];
            CodeExpression[] parms2 = new CodeExpression[2];

            //set up references to variables and data types
            CodeTypeReference intType;
            intType = new CodeTypeReference(typeof(int));
            CodeTypeReferenceExpression intRef;
            intRef = new CodeTypeReferenceExpression(intType);
            CodeTypeReference dateType;
            dateType = new CodeTypeReference(typeof(DateTime));
            CodeTypeReferenceExpression dateRef;
            dateRef = new CodeTypeReferenceExpression(dateType);
            CodeTypeReference decimalType;
            decimalType = new CodeTypeReference(typeof(Decimal));
            CodeTypeReferenceExpression decimalRef;
            decimalRef = new CodeTypeReferenceExpression(decimalType);
            CodeTypeReference doubleType;
            doubleType = new CodeTypeReference(typeof(Double));
            CodeTypeReferenceExpression doubleRef;
            doubleRef = new CodeTypeReferenceExpression(doubleType);

            //System.Windows.Forms.MessageBox.Show("2"); 
            string senderName = "sender";
            CodeTypeReference senderType;
            senderType = new CodeTypeReference(typeof(Object));
            
            string resultName = "result";
            CodeTypeReference resultType;
            resultType = new CodeTypeReference(typeof(int));
            CodeVariableReferenceExpression resultRef;
            resultRef = new CodeVariableReferenceExpression(resultName);

                
            string argsName = "args";
            CodeTypeReference argsType;
            argsType = new CodeTypeReference(typeof(PHVValidators.ServerValidateEventArgs));
            CodeVariableReferenceExpression argsRef;
            argsRef = new CodeVariableReferenceExpression(argsName);
            
            string isValidName = "IsValid";
            CodePropertyReferenceExpression isValidRef;
            isValidRef = new CodePropertyReferenceExpression(argsRef, isValidName);

            //create references to args.Value
            string valueName = "Value";
            CodePropertyReferenceExpression argsValue;
            argsValue = new CodePropertyReferenceExpression(argsRef, valueName);
                
            //primitive expressions (also needed later)
            CodePrimitiveExpression connStringName;
            connStringName = new CodePrimitiveExpression(ConnectionName);
            CodePrimitiveExpression falseValue;
            falseValue = new CodePrimitiveExpression(false);
            CodePrimitiveExpression trueValue;
            trueValue = new CodePrimitiveExpression(true);
            CodePrimitiveExpression zeroValue;
            zeroValue = new CodePrimitiveExpression(0);
            CodePrimitiveExpression parmName = null;
            CodePrimitiveExpression select = null;
            //System.Windows.Forms.MessageBox.Show("3");

            //retrieve connection strings and, based on the connection string, set 
            //db-related values
            XmlNode connNd = GetConnectionElement(pi, ConnectionName);
            if (connNd == null)
            {
                return "";
            }
            //System.Windows.Forms.MessageBox.Show("3a");
            string pName = "";
            try
            {
                XmlAttribute connAtt = connNd.Attributes["providerName"];
                pName = connAtt.Value;
            }
            catch { }
            
            connParms cp;
            switch (pName)
            {
                case "System.Data.SqlClient":
                    cp = SqlClient(DType);
                    break;
                default:
                    cp = SqlClient(DType);
                    break;
            }
            //System.Windows.Forms.MessageBox.Show("4");

            //create a the select statement
            string selectStatement;
            selectStatement = string.Format("Select Count(*) From {0} Where {1} = {2}{1}", TableName, FieldName, cp.parmPrefix);
            select = new CodePrimitiveExpression(selectStatement);
            parmName = new CodePrimitiveExpression(cp.parmPrefix + FieldName);
               
            //create member
            CodeMemberMethod cmm = new CodeMemberMethod();
            cmm.Name = EventName;
            cmm.Attributes = MemberAttributes.Family | MemberAttributes.Final;
            
            //create parameters
            CodeParameterDeclarationExpression sender =
                    new CodeParameterDeclarationExpression(senderType, senderName);
            cmm.Parameters.Add(sender);
            CodeParameterDeclarationExpression args =
              new CodeParameterDeclarationExpression(argsType, argsName);
            cmm.Parameters.Add(args);
            //System.Windows.Forms.MessageBox.Show("5");

            //Declare result
            CodeVariableDeclarationStatement result;
            result = new CodeVariableDeclarationStatement(intType, resultName);
            cmm.Statements.Add(result);

            //declaration for conn object
            string connName = "conn";
            CodeVariableDeclarationStatement conn;
            conn = new CodeVariableDeclarationStatement(cp.connType, connName);
            cmm.Statements.Add(conn);
            CodeVariableReferenceExpression connRef;
            connRef = new CodeVariableReferenceExpression(connName);

            //declaration for cmd object
            string cmdName = "cmd";
            CodeVariableDeclarationStatement cmd;
            cmd = new CodeVariableDeclarationStatement(cp.cmdType, cmdName);
            cmm.Statements.Add(cmd);
            CodeVariableReferenceExpression cmdRef;
            cmdRef = new CodeVariableReferenceExpression(cmdName);


            //System.Windows.Forms.MessageBox.Show("6");
            //create conn object, passing the index into connections strings
            //create a reference to WebConfigurationManager
            CodeTypeReference configMgrRef;
            configMgrRef = new CodeTypeReference(typeof(System.Web.Configuration.WebConfigurationManager));
            CodeTypeReferenceExpression configMgrExp;
            configMgrExp = new CodeTypeReferenceExpression(configMgrRef);

            //create an expression to the ConnectionStrings property
            CodePropertyReferenceExpression connStrings;
            connStrings = new CodePropertyReferenceExpression(configMgrExp, "ConnectionStrings");

            //create an indexed access to the ConnectionStrings property
            parms1[0] = connStringName;
            CodeIndexerExpression configIndex;
            configIndex = new CodeIndexerExpression(connStrings, parms1);
            CodePropertyReferenceExpression connStringAccess;
            connStringAccess = new CodePropertyReferenceExpression(configIndex, "ConnectionString");

            //create the assignment statement for the new Connectionobject, using the indexed access to ConnectionStrings
            CodeObjectCreateExpression connCreate;
            parms1[0] = connStringAccess;
            connCreate = new CodeObjectCreateExpression(cp.connType, parms1);
            CodeAssignStatement connAssign; 
            connAssign = new CodeAssignStatement(connRef, connCreate);
            cmm.Statements.Add(connAssign);

            //Use the Connection object's CreateCommand method  to create the Command object
            CodeMethodInvokeExpression createCommand;

            //System.Windows.Forms.MessageBox.Show("7");
            createCommand = new CodeMethodInvokeExpression(connRef, "CreateCommand");
            CodeAssignStatement cmdAssign;
            cmdAssign = new CodeAssignStatement(cmdRef, createCommand);
            cmm.Statements.Add(cmdAssign);
            
            //set the commandtext property
            CodePropertyReferenceExpression cmdText;
            cmdText = new CodePropertyReferenceExpression(cmdRef, "CommandText");
            CodeAssignStatement cmdTextAssign;
            cmdTextAssign = new CodeAssignStatement(cmdText, select);
            cmm.Statements.Add(cmdTextAssign);

            //Add the parameter 
            CodePropertyReferenceExpression cmdParms;
            cmdParms = new CodePropertyReferenceExpression(cmdRef, "Parameters");
            CodeMethodReferenceExpression cmdParmsAdd;
            cmdParmsAdd = new CodeMethodReferenceExpression(cmdParms,"Add");
            CodeMethodInvokeExpression cmdParmsAddInvoke;
            parms2[0] = parmName;
            parms2[1] = cp.parmType;
            cmdParmsAddInvoke = new CodeMethodInvokeExpression(cmdParmsAdd, parms2);
            cmm.Statements.Add(cmdParmsAddInvoke);

            //System.Windows.Forms.MessageBox.Show("8"); 
            CodeAssignStatement setIsValidFalse;
            setIsValidFalse = new CodeAssignStatement(isValidRef, falseValue);
            
            CodeAssignStatement setIsValidTrue;
            setIsValidTrue = new CodeAssignStatement(isValidRef, trueValue);
                
            //set up Parameters to have value set
            CodeIndexerExpression parmsIndexed;
            parms1[0] = zeroValue;
            parmsIndexed = new CodeIndexerExpression(cmdParms, parms1);
            CodePropertyReferenceExpression parmsIndexedValue;
            parmsIndexedValue = new CodePropertyReferenceExpression(parmsIndexed, "Value");
                
            //convert datatype if value isn't string
            if (DType == "String")
            {
                CodeAssignStatement assignParms;
                assignParms = new CodeAssignStatement(parmsIndexedValue, argsValue);
                cmm.Statements.Add(assignParms);
            }
            else
            {                
                //can't do conversion block
                CodeStatement[] noConvertBlock = new CodeStatement[2];
                noConvertBlock[0] = setIsValidFalse;
                noConvertBlock[1] = new CodeMethodReturnStatement();

                //can do conversion
                CodeStatement[] canConvertBlock = new CodeStatement[1];
                CodeAssignStatement assignParms;
                assignParms = new CodeAssignStatement(parmsIndexedValue, resultRef);
                canConvertBlock[0] = assignParms;

                //create invoke of tryprase on integer type
                CodeMethodReferenceExpression tryParse = null;
                switch(DType)
                {
                    case "Integer" :
                        tryParse = new CodeMethodReferenceExpression(intRef,"TryParse");
                        break;
                    case "DateTime":
                        tryParse = new CodeMethodReferenceExpression(dateRef, "TryParse");
                        break;
                    case "Double":
                        tryParse = new CodeMethodReferenceExpression(doubleRef, "TryParse");
                        break;
                    case "Currency":
                        tryParse = new CodeMethodReferenceExpression(decimalRef, "TryParse");
                        break;
                }

                //System.Windows.Forms.MessageBox.Show("9");
                //invoke tryparse, passing args.Value, and result
                //build args
                CodeDirectionExpression outResultRef;                
                outResultRef = new CodeDirectionExpression(FieldDirection.Out,resultRef);
                parms2[0] = argsValue;
                parms2[1] = outResultRef;
                
                //invoke tryparse
                CodeMethodInvokeExpression tryParseInvoke;
                tryParseInvoke = new CodeMethodInvokeExpression(tryParse,parms2);
                CodeBinaryOperatorExpression testTryParse;
                testTryParse = new CodeBinaryOperatorExpression(tryParseInvoke, CodeBinaryOperatorType.ValueEquality, falseValue);
                //create if
                CodeConditionStatement ccs;
                ccs = new CodeConditionStatement(testTryParse, noConvertBlock, canConvertBlock);
                cmm.Statements.Add(ccs);                
            }

            //open the connection
            CodeMethodInvokeExpression connOpen;
            connOpen = new CodeMethodInvokeExpression(connRef, "Open");
            cmm.Statements.Add(connOpen);

            //Call executescalar and cast to integer
            CodeMethodInvokeExpression executeScalar;
            executeScalar = new CodeMethodInvokeExpression(cmdRef, "ExecuteScalar");
            CodeCastExpression castScalar;
            castScalar = new CodeCastExpression(intType,executeScalar);
            CodeAssignStatement setResult;
            setResult = new CodeAssignStatement(resultRef,castScalar);
            cmm.Statements.Add(setResult);
            //System.Windows.Forms.MessageBox.Show("A");

            //close the connection
            CodeMethodInvokeExpression connClose;
            connClose = new CodeMethodInvokeExpression(connRef, "Close");
            cmm.Statements.Add(connClose);

            //0 returned
            CodeStatement[] isNotValidBlock = new CodeStatement[1];
            isNotValidBlock[0] = setIsValidFalse;
                
           //1 returned
            CodeStatement[] isValidBlock = new CodeStatement[1];
            isValidBlock[0] = setIsValidTrue;
            
           //create test
            CodeBinaryOperatorExpression testResult;
            testResult = new CodeBinaryOperatorExpression(resultRef,CodeBinaryOperatorType.ValueEquality, zeroValue);
                
           CodeConditionStatement ccs2;
           ccs2 = new CodeConditionStatement(testResult, isNotValidBlock, isValidBlock);
           cmm.Statements.Add(ccs2);

           //System.Windows.Forms.MessageBox.Show("B");
           return CodeGeneration(cmm, pi);
        }

        string CodeGeneration(CodeMemberMethod cmm, ProjectInfo pi)
        {
            StringBuilder generatedCode;
            System.IO.StringWriter codeWriter;

            generatedCode = new StringBuilder();
            codeWriter = new System.IO.StringWriter(generatedCode);
            switch (pi.ProjectLanguage)
            {
                case ("CSharp"):
                    Microsoft.CSharp.CSharpCodeProvider csProv = new Microsoft.CSharp.CSharpCodeProvider();
                    csProv.GenerateCodeFromMember(cmm, codeWriter, null);
                    break;
                case ("VisualBasic"):
                    Microsoft.VisualBasic.VBCodeProvider vbProv = new Microsoft.VisualBasic.VBCodeProvider();
                    vbProv.GenerateCodeFromMember(cmm, codeWriter, null);
                    break;
               
            }
            return generatedCode.ToString();
        }

        XmlNode GetConnectionElement(ProjectInfo pi, string ConnectionName)
        {
            ProjectItem cfg;

            try
            {
                cfg = pi.Project.ProjectItems.Item("web.config");
            }
            catch
            {
                cgu.WriteOutput("No config file", GenerationLevel.severe);
                return null;
            }
            XmlDocument xdoc;
            xdoc = new System.Xml.XmlDocument();
            xdoc.Load(@cfg.Properties.Item("FullPath").Value.ToString());
            XmlNode nd =
                xdoc.SelectSingleNode("//connectionStrings/add[@name='" + ConnectionName + "']");
            if (nd == null)
            {
                cgu.WriteOutput("No connection string with the name " + ConnectionName + " found.", GenerationLevel.severe);
                return null;
            }
            return nd;
        }

        void AddCode(string code, ProjectItem CodeFile)
        {
            CodeClass cc = null;
            FileCodeModel fcm = CodeFile.FileCodeModel;
            foreach (CodeElement ce in fcm.CodeElements)
            {
                switch (ce.Kind)
                {
                 case vsCMElement.vsCMElementNamespace:
                 {
                    if (ce.Name.IndexOf(".") > 0)
                    {
                        ce.Name = ce.Name.Substring(0, ce.Name.IndexOf("."));
                    }
                    foreach (CodeElement ce2 in ce.Children)
                    {
                        if (ce2.Kind == vsCMElement.vsCMElementClass)
                        {
                            cc = (CodeClass) ce2;
                        }
                    }
                    break;
                 }
                 case vsCMElement.vsCMElementClass:
                 {
                  cc = (CodeClass) ce;
                  break;
                 }
                }
            }
            
            foreach (CodeElement md in cc.Children)
            {
                if (md.Kind == vsCMElement.vsCMElementFunction)
                {
                    if (code.Contains(md.Name))
                    {
                        cc.RemoveMember(md);
                    }
                }
            }
            EditPoint codePoint = cc.GetEndPoint(vsCMPart.vsCMPartBody).CreateEditPoint();
            codePoint.Indent(codePoint, 0);
            codePoint.Insert(code.Trim() + " \r\n ");
        }

        connParms SqlClient(string DType)
        {
            connParms cp = new connParms();

            //System.Windows.Forms.MessageBox.Show("3a2");
            cp.connType = new CodeTypeReference(typeof(System.Data.SqlClient.SqlConnection));
            cp.cmdType = new CodeTypeReference(typeof(System.Data.SqlClient.SqlCommand));
            cp.parmPrefix = "@";
            //System.Windows.Forms.MessageBox.Show("3a3");            
            CodeTypeReferenceExpression sqlDbType = new CodeTypeReferenceExpression(typeof(SqlDbType));
            //System.Windows.Forms.MessageBox.Show("3b");
            
            switch (DType)
            {                   
               case "Integer":            
                    cp.parmType =  new CodeFieldReferenceExpression(sqlDbType,"Int");
                    break;
                case "String":
                    cp.parmType = new CodeFieldReferenceExpression(sqlDbType, "NText");
                    break;
                case "DateTime":
                    cp.parmType = new CodeFieldReferenceExpression(sqlDbType, "DateTime");
                    break;
                case "Double":
                    cp.parmType = new CodeFieldReferenceExpression(sqlDbType, "BigInt");
                    break;
                case "Currency":
                    cp.parmType = new CodeFieldReferenceExpression(sqlDbType, "Decimal");
                    break;
            }
            return cp;
        }

        Classes FindClass(FileCodeModel fcm)
        {
            Classes cls = new Classes();
            //System.Windows.Forms.MessageBox.Show("Find class");
            cls.lookupClass = null;
            cls.masterClass = null;
            bool bolNotFound = false;
            foreach (CodeElement ce in fcm.CodeElements)
            {
                if (ce.Kind == vsCMElement.vsCMElementClass)
                {
                    if (bolNotFound)
                    {
                        cls.lookupClass = (CodeClass)ce;
                        bolNotFound = false;
                    }
                    else
                    {
                        cls.masterClass = (CodeClass)ce;
                        bolNotFound = true;
                    }
                }
            }
            return cls;
        }
     }
    }

